library ABuffItem requires Table, ABuff, optional StatusEvents

//*****************************************************************
//*  ABUFF ITEM 1.7
//*
//*  written by: Anitarf
//*  requires: -Table
//*            -ABuff
//*
//*  This library automates the use of aBuffs as passive effects of
//*  items. Once an aBuffType is linked to an item type using the
//*  NewABuffItem function, a buff of that type will be
//*  automatically applied on any hero that picks up an item of 
//*  that type as well as any hero that revives carrying that item.
//*  If you have the StatusEvents library in your map, then this
//*  library will also detect reincarnation events and correctly
//*  reapply the buff on those events as well. It will also remove
//*  the buff(s) from the hero when he drops the item.
//*
//*  To define an aBuff item, use this function:
//*
//*    function NewABuffItem takes integer itemTypeId, aBuffType buffType returns nothing
//*
//*  The function only accepts buff types in the STACKING and
//*  REFCOUNT categories.
//*****************************************************************

    private struct itype
        integer itemTypeId
        aBuffType buffType
        
        private static Table table
        static method create takes integer itemTypeId, aBuffType buffType returns itype
            local itype this=itype.allocate()
            set .itemTypeId=itemTypeId
            set .buffType=buffType
            debug if itype.table[itemTypeId]!=0 then
            debug    call BJDebugMsg("NewABuffItem error: item rawcode already associated with an ABuffItem!") 
            debug else
            set itype.table[itemTypeId]=this
            debug endif
            return this
        endmethod
        static method get takes integer itemTypeId returns itype
            return itype.table[itemTypeId]
        endmethod

        private static method pickup takes nothing returns boolean
            local itype this = itype.get(GetItemTypeId(GetManipulatedItem()))
            if this!=0 then
                call ABuffApply(.buffType, GetTriggerUnit(), null, 0.0, 1, 0)
            endif
            return false
        endmethod
        
        private static method ressurect takes unit u returns nothing
            local integer i=0
            local itype this
            loop
                exitwhen i>5
                set this = itype.get(GetItemTypeId(UnitItemInSlot(u, i)))
                if this!=0 then
                    call ABuffRemove(GetABuffFromUnitByType(u, .buffType))
                    call ABuffApply(.buffType, u, null, 0.0, 1, 0)
                endif
                set i=i+1
            endloop
        endmethod
        private static method revive takes nothing returns boolean
            call itype.ressurect(GetTriggerUnit())
            return false
        endmethod

        private static method drop takes nothing returns boolean
            local itype this = itype.get(GetItemTypeId(GetManipulatedItem()))
            if this!=0 then
                call ABuffRemove(GetABuffFromUnitByType(GetTriggerUnit(), .buffType))
            endif
            return false
        endmethod

        static method onInit takes nothing returns nothing
            local trigger t
            set t=CreateTrigger(  )
            call TriggerRegisterAnyUnitEventBJ( t, EVENT_PLAYER_UNIT_PICKUP_ITEM )
            call TriggerAddCondition( t, Condition(function itype.pickup) )
            set t = CreateTrigger(  )
            call TriggerRegisterAnyUnitEventBJ( t, EVENT_PLAYER_UNIT_DROP_ITEM )
            call TriggerAddCondition( t, Condition(function itype.drop) )
          static if LIBRARY_StatusEvents then
            call OnUnitResurrect(itype.ressurect)
          else
            set t=CreateTrigger(  )
            call TriggerRegisterAnyUnitEventBJ( t, EVENT_PLAYER_HERO_REVIVE_FINISH )
            call TriggerAddCondition( t, Condition(function itype.revive) )
          endif

            set itype.table=Table.create()
        endmethod
    endstruct

// ================================================================
//  MAIN USER FUNCTION

    function NewABuffItem takes integer itemTypeId, aBuffType buffType returns nothing
        if buffType.category==ABuff_STACKING or buffType.category==ABuff_REFCOUNT then
            call itype.create(itemTypeId, buffType)
        debug else
            debug call BJDebugMsg("NewABuffItem error: the function only accepts buff types in the STACKING and REFCOUNT categories.")
        endif
    endfunction
endlibrary